﻿/*
File Servant is Copyright (c) 2010, Martin Karlsen. All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are
permitted provided that the following conditions are met:

   1. Redistributions of source code must retain the above copyright notice, this list of
      conditions and the following disclaimer.

   2. Redistributions in binary form must reproduce the above copyright notice, this list
      of conditions and the following disclaimer in the documentation and/or other materials
      provided with the distribution.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS
OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR 
TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;
using System.Net.Sockets;
using System.Net;

namespace FileServant
{
    public class Log
    {
        private static bool opened = false;
        private static Object token = "lock";

        public static String LogFileName()
        {
            String appDataDir = Environment.GetFolderPath(Environment.SpecialFolder.ApplicationData);
            return appDataDir + "\\" + Util.verbCanonicalName + "-logfile.txt";
        }

        private static void WriteLine(String s) 
        {
            lock (token) //ensure that log messages emitted simultaneously gets added in some linear order (and not overwriting eachother)
            {
                try
                {
                    FileInfo logFile = new FileInfo(LogFileName());
                    StreamWriter logWriter = new StreamWriter(logFile.Open(FileMode.Append, FileAccess.Write, FileShare.ReadWrite));
                    if (!opened)
                        logWriter.WriteLine(Time() + "SERVICE ==initialized==");
                    opened = true;
                    logWriter.WriteLine(s);
                    logWriter.Flush();
                    logWriter.Close();
                }
                catch (Exception) { }
            }
        }

        private static String Time()
        {
            DateTime time = DateTime.Now;
            return time.ToShortDateString() + " " + time.ToLongTimeString() + "," + time.ToString("fff") + " ";
        }

        private static String Host(IPAddress ip)
        {
            String host = "<unknown-host>";
            try
            {
                host = Dns.GetHostEntry(ip).HostName;
            }
            catch (Exception) { }
            return host + "(" + ip.ToString() + ") ";
        }

        //
        public static void FileServe(IPAddress client, FileStore.VirtualFile file)
        {
            WriteLine(Time() + "FILESYSTEM " + Host(client) + "READ_FILE URL=" + file.virtualPath + " Path=" + file.theFile.FullName);
        }

        //
        public static void FileWrite(IPAddress client, FileStore.VirtualFile file)
        {
            WriteLine(Time() + "FILESYSTEM " + Host(client) + "WRITE_FILE URL=" + file.virtualPath + " Path=" + file.theFile.FullName);
        }

        //
        public static void DirServe(IPAddress client, FileStore.VirtualFile dir)
        {
            if (dir.theFile == null)
            {
                WriteLine(Time() + "FILESYSTEM " + Host(client) + "LIST_SHARES URL=" + dir.virtualPath);
            }
            else
            {
                WriteLine(Time() + "FILESYSTEM " + Host(client) + "LIST_DIR URL=" + dir.virtualPath + " Path=" + dir.theFile.FullName);
            }
        }

        //
        public static void HttpRequest(IPAddress client, String url, HttpServer.RequestMethod method, String user, String pass)
        {
            WriteLine(Time() + "HTTP " + Host(client) + "HTTP-REQUEST Method=" + method + " URL=" + url + (user != null ? " User=" + user : "") + (pass != null ? " Password=" + pass : ""));
        }

        //
        public static void FtpConnected(IPAddress client, String user, String pass)
        {
            WriteLine(Time() + "FTP " + Host(client) + "CONNECTED User=" + user + " Password=" + pass);
        }

        //
        public static void FtpClosed(IPAddress client, String user, String pass)
        {
            WriteLine(Time() + "FTP " + Host(client) + "CLOSED User=" + user + " Password=" + pass);
        }

        //
        public static void FtpOnline(FtpServer ftpServer)
        {
            String protocol = ftpServer.addressFamily == AddressFamily.InterNetwork ? "IPv4" : "IPv6";
            WriteLine(Time() + "SERVER FTP ONLINE protocol=" + protocol + " port=" + ftpServer.port);
        }

        //
        public static void HttpOnline(HttpServer httpServer)
        {
            String protocol = httpServer.addressFamily == AddressFamily.InterNetwork ? "IPv4" : "IPv6";
            WriteLine(Time() + "SERVER HTTP ONLINE protocol=" + protocol + " port=" + httpServer.port);
        }

        //
        public static void FtpOffline(FtpServer ftpServer)
        {
            String protocol = ftpServer.addressFamily == AddressFamily.InterNetwork ? "IPv4" : "IPv6";
            WriteLine(Time() + "SERVER FTP OFFLINE protocol=" + protocol);
        }

        //
        public static void HttpOffline(HttpServer httpServer)
        {
            String protocol = httpServer.addressFamily == AddressFamily.InterNetwork ? "IPv4" : "IPv6";
            WriteLine(Time() + "SERVER HTTP OFFLINE protocol=" + protocol);
        }

        //
        public static void CreatedShare(FileStore.RootShare share)
        {
            WriteLine(Time() + "SHARE CREATED " + share.rootDirName + (share.user != null ? " username=" + share.user : "") + (share.pass != null ? " password=" + share.pass : ""));
        }

        //
        public static void DeletedShare(FileStore.RootShare share)
        {
            WriteLine(Time() + "SHARE REMOVED " + share.rootDirName + (share.user != null ? " username=" + share.user : "") + (share.pass != null ? " password=" + share.pass : ""));
        }

        public static void Close()
        {
            if (opened)
            {
                WriteLine(Time() + "SERVICE ==TERMINATED==");
                opened = false;
            }
        }
    }
}
