﻿/*
File Servant is Copyright (c) 2010, Martin Karlsen. All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are
permitted provided that the following conditions are met:

   1. Redistributions of source code must retain the above copyright notice, this list of
      conditions and the following disclaimer.

   2. Redistributions in binary form must reproduce the above copyright notice, this list
      of conditions and the following disclaimer in the documentation and/or other materials
      provided with the distribution.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS
OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR 
TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Reflection;
using System.IO;
using System.Threading;
using System.Net;

namespace FileServant
{
    class UpdateCheck
    {
        private static String updateServer = "http://www.fileservant.net/download/latestversion.txt";
        private volatile bool checkDone = false;

        private Version myVersion = Assembly.GetExecutingAssembly().GetName().Version;
        private Version onlineVersion = null;
        private OnlineVersion oVersion = null;

        public UpdateCheck(bool smallWait)
        {
            String appDataDir = Environment.GetFolderPath(Environment.SpecialFolder.ApplicationData);
            String updateFileName = appDataDir + "\\" + Util.verbCanonicalName + "-updatecheck.txt";
            try
            {
                FileInfo updateFile = new FileInfo(updateFileName);
                if (updateFile.Exists && updateFile.LastWriteTime.DayOfYear == DateTime.Now.DayOfYear)
                {
                    //read updateinformation from file
                    StreamReader reader = new StreamReader(updateFile.OpenRead());
                    String updateVer = reader.ReadLine();
                    String updateUrl = reader.ReadLine();
                    reader.Close();
                    oVersion = new OnlineVersion(updateVer, updateUrl);
                    if (oVersion == null)
                        onlineVersion = myVersion;
                    else
                        onlineVersion = new Version(oVersion.versionNumber);
                    checkDone = true;
                }
                else
                {
                    //perform an update check asynchronously
                    Thread updateCheckThread = new Thread(new ThreadStart(this.UpdateCheckThread));
                    updateCheckThread.Start();
                    if (smallWait)
                    {
                        //wait up to 3 seconds for update server to respond before continuing
                        for (int i = 0; i < 30; i++)
                        {
                            Thread.Sleep(100);
                            if (CheckIsDone())
                                break;
                        }
                    }
                }
            }
            catch (Exception) { }
        }


        private void UpdateCheckThread()
        {
            String appDataDir = Environment.GetFolderPath(Environment.SpecialFolder.ApplicationData);
            String updateFileName = appDataDir + "\\" + Util.verbCanonicalName + "-updatecheck.txt";
            try
            {
                FileInfo updateFile = new FileInfo(updateFileName);
                //perform new update check
                oVersion = CheckOnline();
                if (oVersion != null)
                {
                    //write update information to file - this way we dont bombard update server more than once a day
                    StreamWriter writer = new StreamWriter(updateFile.Open(FileMode.Create, FileAccess.Write, FileShare.ReadWrite));
                    writer.WriteLine(oVersion.versionNumber);
                    writer.WriteLine(oVersion.url);
                    writer.Flush();
                    writer.Close();
                }
            }
            catch (Exception) { }
            if (oVersion == null)
                onlineVersion = myVersion;
            else
                onlineVersion = new Version(oVersion.versionNumber);
            checkDone = true;
        }


        public bool CheckIsDone()
        {
            return checkDone;
        }


        private class OnlineVersion
        {
            public OnlineVersion(String argVerNum, String argUrl)
            {
                versionNumber = argVerNum;
                url = argUrl;
            }
            public String versionNumber;
            public String url;
        }

        private OnlineVersion CheckOnline()
        {
            //perform online check
            try
            {
                HttpWebRequest req = (HttpWebRequest)WebRequest.Create(updateServer);
                req.Method = "GET";
                WebResponse response = req.GetResponse();
                StreamReader reader = new StreamReader(response.GetResponseStream(), System.Text.Encoding.UTF8);
                String updateVer = reader.ReadLine();
                String updateUrl = reader.ReadLine();
                reader.Close();
                response.Close();
                if (updateVer != null && updateUrl != null)
                    return new OnlineVersion(updateVer, updateUrl);
            }
            catch (Exception) { }
            return null;
        }

        public bool OnlineVersionAvailable()
        {
            if (!checkDone)
                return false;
            return myVersion < onlineVersion;
        }

        public String GetOnlineVersionURL()
        {
            if (oVersion == null)
                return null;
            return oVersion.url;
        }

        public Version GetCurrentVersion()
        {
            return myVersion;
        }

        public Version GetOnlineVersion()
        {
            return onlineVersion;
        }
    }
}
