﻿/*
File Servant is Copyright (c) 2010, Martin Karlsen. All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are
permitted provided that the following conditions are met:

   1. Redistributions of source code must retain the above copyright notice, this list of
      conditions and the following disclaimer.

   2. Redistributions in binary form must reproduce the above copyright notice, this list
      of conditions and the following disclaimer in the documentation and/or other materials
      provided with the distribution.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS
OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR 
TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;
using Microsoft.Win32;
using Microsoft.Win32.SafeHandles;
using System.Runtime.InteropServices;
using System.Diagnostics;

namespace FileServant
{
    class Util
    {
        public static String toolName = "File Servant";
        public static String toolPipeName = "FileServantPipe";
        public static String verbCanonicalName = "FileServant";

        public static String toolNameLink = toolName + " [www.FileServant.net]";
        public static String toolNameHtmlLink = toolName + " [<a href=\"http://www.fileservant.net/\">www.FileServant.net</a>]";

        //read a line from a stream
        public static String ReadLineFromStream(Stream readFrom)
        {
            int byteRead = readFrom.ReadByte();
            List<byte> bytes = new List<byte>();
            while (byteRead >= 0)
            {
                byte b = (byte)byteRead;
                char c = (char)byteRead;
                if (c == '\n')
                    break;
                if (c != '\r')
                    bytes.Add(b);
                byteRead = readFrom.ReadByte();
            }
            return Encoding.UTF8.GetString(bytes.ToArray());
        }

        public static String IPToString(System.Net.IPAddress ip)
        {
            if (ip.AddressFamily == System.Net.Sockets.AddressFamily.InterNetworkV6)
            {
                //ipv6 - remove scopeid from string
                String scope = "%" + ip.ScopeId;
                String result = ip.ToString();
                result = result.Replace(scope, "");
                return result;
            }
            else
            {
                //ipv4
                return ip.ToString();
            }
        }

        public static byte[] StringToUTF8Bytes(String s)
        {
            byte[] bytes = Encoding.UTF8.GetBytes(s);
            return bytes;
        }


        public static StreamWriter GetStreamWriter(Stream s) {
            return new StreamWriter(s);
        }

        public static String UrlEncode(String s)
        {
            String result = Uri.EscapeDataString(s);
            return result.Replace("%2F", "/");
        }


        //exercise the win32 api to acquire file handles for directories
        public const uint GENERIC_READ = 0x80000000;
        public const uint FILE_SHARE_READ = 0x00000001;
        public const uint FILE_SHARE_WRITE = 0x00000002;
        public const uint OPEN_EXISTING = 3;
        public const uint FILE_FLAG_BACKUP_SEMANTICS = 0x02000000;
        public const uint FILE_ATTRIBUTE_READONLY = 1;

        [DllImport("kernel32.dll", SetLastError = true, CharSet = CharSet.Unicode)]
        public static extern SafeFileHandle CreateFile(string lpFileName, uint dwDesiredAccess,
                                                  uint dwShareMode, IntPtr lpSecurityAttributes,
                                                  uint dwCreationDisposition, uint dwFlagsAndAttributes,
                                                  IntPtr hTemplateFile);



    }
}
