﻿/*
File Servant is Copyright (c) 2010, Martin Karlsen. All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are
permitted provided that the following conditions are met:

   1. Redistributions of source code must retain the above copyright notice, this list of
      conditions and the following disclaimer.

   2. Redistributions in binary form must reproduce the above copyright notice, this list
      of conditions and the following disclaimer in the documentation and/or other materials
      provided with the distribution.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS
OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR 
TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/
using System;
using System.Collections;
using System.Collections.Generic;
using System.ComponentModel;
using System.Configuration.Install;
using System.Linq;
using Microsoft.Win32;



namespace InstallerActions
{
    [RunInstaller(true)]
    public partial class SetupCustomActions : System.Configuration.Install.Installer
    {
        private String GetInstallDir()
        {
            String path = Context.Parameters["assemblypath"];
            return path.Substring(0, path.LastIndexOf('\\'));
        }


        public SetupCustomActions()
        {
            InitializeComponent();
        }

        
        [System.Security.Permissions.SecurityPermission(System.Security.Permissions.SecurityAction.Demand)]
        public override void Install(IDictionary stateSaver)
        {
            base.Install(stateSaver);
        }


        [System.Security.Permissions.SecurityPermission(System.Security.Permissions.SecurityAction.Demand)]
        public override void Commit(IDictionary savedState)
        {
            base.Commit(savedState);
            //add to system path environment
            String installDir = GetInstallDir();
            bool success = SetupUtil.EnsureValueInSystemPathEnvironment(true, installDir);
            if (!success)
                System.Windows.Forms.MessageBox.Show("Setup could not add path to " + SetupUtil.exe + " to the system path environment variable.\nYou might have to manually add this in order for " + SetupUtil.toolName + " to function properly.");

            String shellx86Path = GetInstallDir() + "\\Shellx86.dll";
            String shellx64Path = GetInstallDir() + "\\Shellx64.dll";

            bool result = SetupUtil.PurgeFirstBytesFromFile(shellx86Path, 10);
            
            result = SetupUtil.PurgeFirstBytesFromFile(shellx64Path, 10);

            //register x86 or x64 shell extension depending on OS type
            if (Environment.Is64BitOperatingSystem)
                success = SetupUtil.RegAsm(shellx64Path, true);
            else
                success = SetupUtil.RegAsm(shellx86Path, true);

            if (!success)
                System.Windows.Forms.MessageBox.Show("Setup could not install explorer shell extension.\n" + SetupUtil.toolName + " will not be available through right-click on folder. Sorry.");

            //register firewall exception for this application
            success = false;
            String exePath = GetInstallDir() + "\\" + SetupUtil.exe;
            SetupUtil.FirewallType firewallType = SetupUtil.GetFirewallType();

            if (firewallType == SetupUtil.FirewallType.xp)
                success = SetupUtil.XPFirewallAdd(SetupUtil.toolName, exePath);
            else if (firewallType == SetupUtil.FirewallType.win7)
                success = SetupUtil.Win7FirewallAdd(SetupUtil.toolName, exePath);
            if (!success)
                System.Windows.Forms.MessageBox.Show("Setup could not modify firewall settings.\nYou might have to manually add a rule to the firewall for this application.");
        }


        [System.Security.Permissions.SecurityPermission(System.Security.Permissions.SecurityAction.Demand)]
        public override void Rollback(IDictionary savedState)
        {
            base.Rollback(savedState);
        }


        [System.Security.Permissions.SecurityPermission(System.Security.Permissions.SecurityAction.Demand)]
        public override void Uninstall(IDictionary savedState)
        {
            base.Uninstall(savedState);

            //remove from system path environment
            String installDir = GetInstallDir();
            bool success = SetupUtil.EnsureValueInSystemPathEnvironment(false, installDir);
            if (!success)
                System.Windows.Forms.MessageBox.Show("Setup could not remove path to " + SetupUtil.exe + " from the system path environment variable.");

            String shellx86Path = GetInstallDir() + "\\Shellx86.dll";
            String shellx64Path = GetInstallDir() + "\\Shellx64.dll";

            //un-register x86 or x64 shell extension depending on OS type
            if (Environment.Is64BitOperatingSystem)
                success = SetupUtil.RegAsm(shellx64Path, false);
            else
                success = SetupUtil.RegAsm(shellx86Path, false);

            if (!success)
                System.Windows.Forms.MessageBox.Show("Setup encountered an error when unregisterring the " + SetupUtil.toolName + " shell extension.");

            //un-register firewall exception for this application
            success = false;
            String exePath = GetInstallDir() + "\\" + SetupUtil.exe;
            SetupUtil.FirewallType firewallType = SetupUtil.GetFirewallType();
            if (firewallType == SetupUtil.FirewallType.xp)
                success = SetupUtil.XPFirewallRemove(SetupUtil.toolName);
            else if (firewallType == SetupUtil.FirewallType.win7)
                success = SetupUtil.Win7FirewallRemove(SetupUtil.toolName);
            if (!success)
                System.Windows.Forms.MessageBox.Show("Setup could not modify firewall settings to clean up rule for uninstalled application.");
        }

    }
}
