﻿/*
File Servant is Copyright (c) 2010, Martin Karlsen. All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are
permitted provided that the following conditions are met:

   1. Redistributions of source code must retain the above copyright notice, this list of
      conditions and the following disclaimer.

   2. Redistributions in binary form must reproduce the above copyright notice, this list
      of conditions and the following disclaimer in the documentation and/or other materials
      provided with the distribution.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS
OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR 
TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;
using Microsoft.Win32;
using Microsoft.Win32.SafeHandles;
using System.Runtime.InteropServices;
using System.Reflection;
using System.Diagnostics;

namespace InstallerActions
{
    class SetupUtil
    {
        public static String toolName = "File Servant";
        public static String exe = "FileServant.exe";


        public enum FirewallType
        {
            xp,
            win7,
            error
        }


        public enum SendMessageTimeoutFlags : uint
        {
            SMTO_NORMAL = 0x0000,
            SMTO_BLOCK = 0x0001,
            SMTO_ABORTIFHUNG = 0x0002,
            SMTO_NOTIMEOUTIFNOTHUNG = 0x0008
        }


        // Import library used to broadcast a system message that the environment changed.
        [DllImport("user32.dll", SetLastError = true, CharSet = CharSet.Auto)]
        static extern IntPtr SendMessageTimeout(IntPtr hWnd, uint Msg, UIntPtr wParam, string lParam, SendMessageTimeoutFlags fuFlags, uint uTimeout, out UIntPtr lpdwResult);


        public static bool RegAsm(String exePath, bool register)
        {
            try
            {
                if (File.Exists(exePath))
                {
                    String windir = Environment.GetEnvironmentVariable("systemroot", EnvironmentVariableTarget.Machine);
                    if (windir == null)
                        windir = Environment.GetEnvironmentVariable("windir", EnvironmentVariableTarget.Machine);

                    String regasm = windir + (exePath.ToLower().EndsWith("x64.dll") ? "\\Microsoft.NET\\Framework64\\v4.0.30319\\RegAsm.exe" : "\\Microsoft.NET\\Framework\\v4.0.30319\\RegAsm.exe");

                    if (File.Exists(regasm))
                    {
                        String argument = register ? " /codebase" : " /unregister";
                        //invoke regasm
                        Process p = new Process();
                        p.StartInfo.FileName = regasm;
                        p.StartInfo.Arguments = " \"" + exePath + "\" " + argument;
                        //p.StartInfo.Arguments = " /?";
                        p.StartInfo.CreateNoWindow = true;
                        p.StartInfo.UseShellExecute = false;
                        p.StartInfo.RedirectStandardOutput = true;
                        p.Start();
                        String result = p.StandardOutput.ReadToEnd();
                        p.WaitForExit();
                        p.Close();
                        return (result.Contains(toolName + " registerring") || result.Contains(toolName + " un-registerring"));
                    }
                    else
                        return false;
                }
                return false;
            }
            catch (Exception ex)
            {
                System.Windows.Forms.MessageBox.Show(ex.Message + "\n" + ex.Source + "\n" + ex.StackTrace);
                return false;
            }
        }


        public static bool PurgeFirstBytesFromFile(String file, int numBytes)
        {
            try
            {
                //read file
                FileInfo f = new FileInfo(file);
                byte[] fileBytes = new byte[f.Length];
                Stream s = f.OpenRead();
                int readBytes = 0;
                while (true)
                {
                    readBytes = s.Read(fileBytes, readBytes, fileBytes.Length - readBytes);
                    if (readBytes == 0)
                        break;
                }
                s.Close();
                byte firstByte = fileBytes[0];

                //protect againts multiple invocations - only purge if first n bytes are identical
                for (int i = 1; i < numBytes; i++)
                {
                    if (fileBytes[i] != firstByte)
                        return true;
                }

                //purge
                s = f.OpenWrite();
                s.SetLength(0);
                s.Write(fileBytes, 10, fileBytes.Length - 10);
                s.Flush();
                s.Close();
                return true;
            }
            catch (Exception ex)
            {
                System.Windows.Forms.MessageBox.Show(ex.Message + "\n" + ex.Source + "\n" + ex.StackTrace);
                return false;
            }
        }



        //ensuring presence or absence of a specific string in the PATH system environment variable 
        //(function invoked by installer)
        public static bool EnsureValueInSystemPathEnvironment(bool shouldBePresent, String value)
        {
            String key = "SYSTEM\\CurrentControlSet\\Control\\Session Manager\\Environment";
            RegistryKey env = Registry.LocalMachine.OpenSubKey(key, RegistryKeyPermissionCheck.ReadWriteSubTree, System.Security.AccessControl.RegistryRights.FullControl);
            String path = (String)env.GetValue("Path", "", RegistryValueOptions.DoNotExpandEnvironmentNames);

            String lowerPath = path.ToLower();
            String lowerValue = value.ToLower();
            bool success = true;
            bool actuallyPresent = lowerPath.EndsWith(lowerValue) ||
                    lowerPath.StartsWith(lowerValue + ";") ||
                    lowerPath.IndexOf(";" + lowerValue + ";") >= 0 ||
                    lowerPath == lowerValue;
            if (shouldBePresent)
            {
                if (!actuallyPresent)
                {
                    //add path to environment
                    if (path.Length == 0)
                        path = value;
                    else
                        path += ";" + value;
                    env.SetValue("Path", path, RegistryValueKind.ExpandString);
                }
                //verify
                String path2 = (String)env.GetValue("Path", "", RegistryValueOptions.DoNotExpandEnvironmentNames);
                success = path == path2;
            }
            else
            {
                if (actuallyPresent)
                {
                    //remove path from environment
                    int startIndex = lowerPath.IndexOf(lowerValue);
                    while (startIndex + lowerValue.Length < lowerPath.Length) //we need to make shure that we are not removing part of a matching path name, check for ';'
                    {
                        if (startIndex + lowerValue.Length < lowerPath.Length)
                        {
                            if (lowerPath[startIndex + lowerValue.Length] == ';')
                                break;
                            startIndex = lowerPath.IndexOf(lowerValue, startIndex + 1);
                        }
                    }
                    path = path.Substring(0, startIndex) + path.Substring(startIndex + lowerValue.Length);
                    path = path.Replace(";;", ";");
                    if (path.StartsWith(";"))
                        path = path.Substring(1);
                    if (path.EndsWith(";"))
                        path = path.Substring(0, path.Length - 1);
                    env.SetValue("Path", path, RegistryValueKind.ExpandString);
                }
                //verify
                String path2 = (String)env.GetValue("Path", "", RegistryValueOptions.DoNotExpandEnvironmentNames);
                success = path2 == path;
            }

            //Notify all windows that User Environment variables are changed
            IntPtr HWND_BROADCAST = (IntPtr)0xffff;
            const UInt32 WM_SETTINGCHANGE = 0x001A;
            UIntPtr result;
            IntPtr settingResult = SendMessageTimeout(HWND_BROADCAST, WM_SETTINGCHANGE, (UIntPtr)0, "Environment", SendMessageTimeoutFlags.SMTO_NORMAL, 10000, out result);
            return success;
        }


        public static FirewallType GetFirewallType()
        {
            String netsh = Environment.SystemDirectory + "\\netsh.exe";
            if (File.Exists(netsh))
            {
                try
                {
                    //query netsh for capabillities
                    Process p = new Process();
                    p.StartInfo.FileName = netsh;
                    p.StartInfo.Arguments = " /?";
                    p.StartInfo.CreateNoWindow = true;
                    p.StartInfo.UseShellExecute = false;
                    p.StartInfo.RedirectStandardOutput = true;
                    p.Start();
                    String result = p.StandardOutput.ReadToEnd();
                    p.WaitForExit();
                    p.Close();
                    if (result.Contains("advfirewall"))
                    {
                        return FirewallType.win7;
                    }
                    return FirewallType.xp;
                }
                catch (Exception ex)
                {
                    System.Windows.Forms.MessageBox.Show(ex.Message + "\n" + ex.Source + "\n" + ex.StackTrace);
                    return FirewallType.error;
                }
            }
            else
                return FirewallType.error;

        }


        public static bool Win7FirewallRemove(String ruleName)
        {
            String netsh = Environment.SystemDirectory + "\\netsh.exe";
            if (File.Exists(netsh))
            {
                try
                {
                    for (int tries = 0; tries < 10; tries++)
                    {
                        //check if rule exists
                        Process p = new Process();
                        p.StartInfo.FileName = netsh;
                        p.StartInfo.Arguments = " advfirewall firewall show rule name=\"" + ruleName + "\"";
                        p.StartInfo.CreateNoWindow = true;
                        p.StartInfo.UseShellExecute = false;
                        p.StartInfo.RedirectStandardOutput = true;
                        p.Start();
                        String result = p.StandardOutput.ReadToEnd();
                        p.WaitForExit();
                        p.Close();
                        if (result.Contains(ruleName))
                        {
                            //delete an existing rule
                            p = new Process();
                            p.StartInfo.FileName = netsh;
                            p.StartInfo.Arguments = " advfirewall firewall delete rule name=\"" + ruleName + "\"";
                            p.StartInfo.CreateNoWindow = true;
                            p.StartInfo.UseShellExecute = false;
                            p.StartInfo.RedirectStandardOutput = true;
                            p.Start();
                            result = p.StandardOutput.ReadToEnd();
                            p.WaitForExit();
                            p.Close();
                        }
                        else
                        {
                            return true;
                        }
                    }
                    return false;
                }
                catch (Exception ex)
                {
                    System.Windows.Forms.MessageBox.Show(ex.Message + "\n" + ex.Source + "\n" + ex.StackTrace);
                    return false;
                }
            }
            else
                return false;
        }

        
        public static bool Win7FirewallAdd(String ruleName, String program)
        {
            if (!Win7FirewallRemove(ruleName)) //first make sure that any existing rule has been deleted
                return false;
            String netsh = Environment.SystemDirectory + "\\netsh.exe";
            if (File.Exists(netsh))
            {
                try
                {
                    //add in-bound rule
                    Process p = new Process();
                    p.StartInfo.FileName = netsh;
                    p.StartInfo.Arguments = " advfirewall firewall add rule name=\"" + ruleName + "\" dir=in action=allow program=\"" + program + "\" enable=yes profile=any";
                    p.StartInfo.CreateNoWindow = true;
                    p.StartInfo.UseShellExecute = false;
                    p.StartInfo.RedirectStandardOutput = true;
                    p.Start();
                    String result = p.StandardOutput.ReadToEnd();
                    p.WaitForExit();
                    p.Close();
                    if (!result.ToLower().Contains("ok"))
                        return false;

                    //add out-bound rule
                    p = new Process();
                    p.StartInfo.FileName = netsh;
                    p.StartInfo.Arguments = " advfirewall firewall add rule name=\"" + ruleName + "\" dir=out action=allow program=\"" + program + "\" enable=yes profile=any";
                    p.StartInfo.CreateNoWindow = true;
                    p.StartInfo.UseShellExecute = false;
                    p.StartInfo.RedirectStandardOutput = true;
                    p.Start();
                    result = p.StandardOutput.ReadToEnd();
                    p.WaitForExit();
                    p.Close();
                    if (!result.ToLower().Contains("ok"))
                        return false;
                    return true;
                }
                catch (Exception ex)
                {
                    System.Windows.Forms.MessageBox.Show(ex.Message + "\n" + ex.Source + "\n" + ex.StackTrace);
                    return false;
                }
            }
            else
                return false;
        }


        public static bool XPFirewallRemove(String ruleName)
        {
            String netsh = Environment.SystemDirectory + "\\netsh.exe";
            if (File.Exists(netsh))
            {
                try
                {
                    for (int tries = 0; tries < 10; tries++)
                    {
                        //check if rule exists
                        Process p = new Process();
                        p.StartInfo.FileName = netsh;
                        p.StartInfo.Arguments = " firewall show allowedprogram";
                        p.StartInfo.CreateNoWindow = true;
                        p.StartInfo.UseShellExecute = false;
                        p.StartInfo.RedirectStandardOutput = true;
                        p.Start();
                        String result = p.StandardOutput.ReadToEnd();
                        p.WaitForExit();
                        p.Close();
                        if (result.Contains(ruleName))
                        {
                            //delete an existing rule
                            int i = result.IndexOf(ruleName);
                            i = result.IndexOf('/', i);
                            result = result.Substring(i + 2);
                            result = result.Substring(0, result.IndexOf('\r'));
                            p = new Process();
                            p.StartInfo.FileName = netsh;
                            p.StartInfo.Arguments = " firewall delete allowedprogram program=\"" + result + "\" profile=ALL";
                            p.StartInfo.CreateNoWindow = true;
                            p.StartInfo.UseShellExecute = false;
                            p.StartInfo.RedirectStandardOutput = true;
                            p.Start();
                            result = p.StandardOutput.ReadToEnd();
                            p.WaitForExit();
                            p.Close();
                        }
                        else
                        {
                            return true;
                        }
                    }
                    return false;
                }
                catch (Exception ex)
                {
                    System.Windows.Forms.MessageBox.Show(ex.Message + "\n" + ex.Source + "\n" + ex.StackTrace);
                    return false;
                }
            }
            else
                return false;
        }

        
        public static bool XPFirewallAdd(String ruleName, String program)
        {
            if (!XPFirewallRemove(ruleName)) //first make sure that any existing rule has been deleted
                return false;
            String netsh = Environment.SystemDirectory + "\\netsh.exe";
            if (File.Exists(netsh))
            {
                try
                {
                    //add rule
                    Process p = new Process();
                    p.StartInfo.FileName = netsh;
                    p.StartInfo.Arguments = " firewall add allowedprogram program=\"" + program + "\" name=\"" + ruleName + "\" profile=ALL mode=ENABLE";
                    p.StartInfo.CreateNoWindow = true;
                    p.StartInfo.UseShellExecute = false;
                    p.StartInfo.RedirectStandardOutput = true;
                    p.Start();
                    String result = p.StandardOutput.ReadToEnd();
                    p.WaitForExit();
                    p.Close();
                    //check rule exists
                    p = new Process();
                    p.StartInfo.FileName = netsh;
                    p.StartInfo.Arguments = " firewall show allowedprogram";
                    p.StartInfo.CreateNoWindow = true;
                    p.StartInfo.UseShellExecute = false;
                    p.StartInfo.RedirectStandardOutput = true;
                    p.Start();
                    result = p.StandardOutput.ReadToEnd();
                    p.WaitForExit();
                    p.Close();
                    if (result.Contains(ruleName))
                        return true;
                    return false;
                }
                catch (Exception ex)
                {
                    System.Windows.Forms.MessageBox.Show(ex.Message + "\n" + ex.Source + "\n" + ex.StackTrace);
                    return false;
                }
            }
            else
                return false;
        }
    }
}
